﻿
Imports System.Runtime.CompilerServices
Imports System.Reflection
Imports Helpers.BitOperation

Namespace System.ComponentModel
   'NotifyPropertyChanged
   Public Class NotifyPropertyChanged : Implements INotifyPropertyChanged, ICloneable


      Private Shared _IsInDesignMode As Boolean? = Nothing
      Public Shared Function IsInDesignMode() As Boolean
         If Not _IsInDesignMode.HasValue Then
            _IsInDesignMode = Process.GetCurrentProcess().ProcessName.StartsWith("devenv", StringComparison.Ordinal)
         End If
         Return _IsInDesignMode.Value
      End Function

      Public Custom Event PropertyChanged As PropertyChangedEventHandler Implements INotifyPropertyChanged.PropertyChanged
         AddHandler(ByVal value As PropertyChangedEventHandler)
            SetPropChangedHandler(value, True)
         End AddHandler
         RemoveHandler(ByVal value As PropertyChangedEventHandler)
            SetPropChangedHandler(value, False)
         End RemoveHandler
         RaiseEvent(ByVal sender As Object, ByVal e As PropertyChangedEventArgs)
            RaisePropChanged(e)
         End RaiseEvent
      End Event

      '#Region "ChangeTracking - noch nicht ausgereift"

      '      Protected Shared _PropNames As String()
      '      Protected ReadOnly Property PropNames As String()
      '         Get
      '            If _PropNames.Null Then
      '               Dim pNames = From p In Me.GetType.GetProperties(Reflection.BindingFlags.Public Or Reflection.BindingFlags.GetProperty Or BindingFlags.Instance) _
      '                  Where (Not p.PropertyType.IsGenericType) AndAlso p.GetIndexParameters.Length = 0 _
      '                  Order By p.Name Select p.Name
      '               _PropNames = pNames.Except({"HasChanges", "RememberHasChanges"}).ToArray
      '            End If
      '            Return _PropNames
      '         End Get
      '      End Property
      '      Private Class ChangeTracking : Inherits WeakAttachBase(Of NotifyPropertyChanged, ChangeTracking)
      '         Friend ChangeFlags As Int32
      '      End Class

      '      Private Sub TrackChange(ByVal sender As Object, ByVal e As PropertyChangedEventArgs)
      '         HasChanges(e.PropertyName) = True
      '      End Sub

      '      Public Property HasChanges(ByVal propName As String) As Boolean
      '         Get
      '            Dim ct As ChangeTracking = Nothing
      '            If ChangeTracking.TryGet(Me, ct) Then
      '               Dim i = 1 << PropNames.IndexOf(propName)
      '               Return ct.ChangeFlags.HasFlag(i)
      '            End If
      '            Return False
      '         End Get
      '         Set(ByVal value As Boolean)
      '            Dim i = 1 << PropNames.IndexOf(propName)
      '            ChangeTracking.Get(Me).ChangeFlags.SetFlag(i, value)
      '         End Set
      '      End Property
      '      Public ReadOnly Property HasChanges As Boolean
      '         Get
      '            Dim ct As ChangeTracking = Nothing
      '            Return If(ChangeTracking.TryGet(Me, ct), ct.ChangeFlags <> 0, False)
      '         End Get
      '      End Property
      '      Public Sub ClearChanges()
      '         ChangeTracking.Remove(Me)
      '      End Sub
      '      Public Property RememberHasChanges As Boolean
      '         Get
      '            Dim pce As PropChangedEvent = Nothing
      '            If Not PropChangedEvent.TryGet(Me, pce) Then Return False
      '            Return pce.PropChangedEvent.GetInvocationList.Contains(DirectCast(AddressOf TrackChange, PropertyChangedEventHandler))
      '         End Get
      '         Set(ByVal value As Boolean)
      '            Dim handler = DirectCast(AddressOf TrackChange, PropertyChangedEventHandler)
      '            With PropChangedEvent.Get(Me)
      '               .PropChangedEvent.Be([Delegate].Remove(.PropChangedEvent, handler))
      '               If value Then .PropChangedEvent.Be([Delegate].Combine(.PropChangedEvent, handler))
      '            End With
      '         End Set
      '      End Property
      '#End Region 'ChangeTracking - noch nicht ausgereift


      Private Class PropChangedEvent : Inherits WeakAttachBase(Of NotifyPropertyChanged, PropChangedEvent)
         Friend PropChangedEvent As PropertyChangedEventHandler
      End Class
      Public Sub SetPropChangedHandler(ByVal handler As PropertyChangedEventHandler, ByVal [addHandler] As Boolean)
         If handler Is Nothing Then Throw New ArgumentNullException("handler")
         With PropChangedEvent.Get(Me)
            If [addHandler] Then
               .PropChangedEvent.Be([Delegate].Combine(.PropChangedEvent, handler))
            Else
               Dim h = [Delegate].Remove(.PropChangedEvent, handler)
               If h.Null Then PropChangedEvent.Remove(Me) Else .PropChangedEvent.Be(h)
            End If
         End With
      End Sub

      Protected Function ChangePropIfDifferent(Of T)(ByVal value As T, ByRef backingField As T, ByVal ParamArray propNames() As String) As Boolean
         Return ChangePropIfDifferent(value, backingField, propNames.ConvertAll(Function(s) New PropertyChangedEventArgs(s)))
      End Function
      Protected Function ChangePropIfDifferent(Of T)(ByVal value As T, ByRef backingField As T, ByVal ParamArray props() As PropertyChangedEventArgs) As Boolean
         If Object.Equals(backingField, value) Then Return False
         backingField = value
         RaisePropChanged(props)
         Return True
      End Function
      Public Sub RaisePropChanged(ByVal ParamArray names() As String)
         RaisePropChanged(names.ConvertAll(Function(name) New PropertyChangedEventArgs(name)))
      End Sub
      Public Sub RaisePropChanged(ByVal ParamArray eventArgses() As PropertyChangedEventArgs)
         RaisePropChanged(DirectCast(eventArgses, IEnumerable(Of PropertyChangedEventArgs)))
      End Sub
      Public Sub RaisePropChanged(ByVal eventArgses As IEnumerable(Of PropertyChangedEventArgs))
         Dim pce As PropChangedEvent = Nothing
         If Not PropChangedEvent.TryGet(Me, pce) Then Return
         Dim handler = pce.PropChangedEvent
         For Each e In eventArgses
            handler.Invoke(Me, e)
         Next
      End Sub

      ''' <summary>Überschreibungen werden auch von ICloneable.Clone() aufgerufen</summary>
      Public Overridable Sub CopyTo(ByVal other As Object)
      End Sub
      Public Function Clone() As Object Implements ICloneable.Clone
         Dim cl = MemberwiseClone()
         CopyTo(cl)
         Return cl
      End Function
   End Class

   'Public Module INotifyPropertyChangedX

   '   Private _ChangedItems As New ConditionalWeakTable(Of Object, Object), _Dummi As Object

   '   <Extension(), DebuggerStepThrough()> _
   '   Public Function IsChanged(Of T As Class)(ByVal itm As T) As Boolean
   '      Return _ChangedItems.TryGetValue(itm, _Dummi)
   '   End Function

   '   <Extension(), DebuggerStepThrough()> _
   '   Public Function IsChanged(Of T As Class)(ByVal itm As T, ByVal value As Boolean) As Boolean
   '      If value Then
   '         IsChanged = value <> _ChangedItems.TryGetValue(itm, _Dummi)
   '         If IsChanged Then _ChangedItems.Add(itm, Nothing)
   '      Else
   '         IsChanged = _ChangedItems.Remove(itm)
   '      End If
   '   End Function

   'End Module

End Namespace
