﻿Namespace System

   ''' <summary>
   ''' eingegeben werden Schwellwerte eines unendlichen Bereiches von IComparable, zusammen mit Items. Die Items matchen dann auf alle Werte von (einschließlich) diesem Schwellwert bis zum nächsthöheren.
   ''' Im Konstruktor ist das Smallest-Item anzugeben, welches auf alles matcht, was kleiner ist als der kleinste Schwellwert.
   ''' Die Klasse kann auch als Mini-Dictionary herhalten. 
   ''' Initialisierung mittm From-Auflistungs-Initialisierer - Reihenfolge egal:
   ''' Private dic As New ThresholdDictionary(Of Double, String)("smaller than 1.0") From {{2, ">= 2.0"}, {3, ">= 3.0"}, {1, ">= 1.0"}}
   ''' </summary>
   Public Class ThresholdDictionary(Of TKey As IComparable, TValue) : Implements IEnumerable(Of TValue)

      Private _Keys As New List(Of TKey)
      Private _Values As New List(Of TValue)

      Public Sub New(ByVal smallest As TValue)
         _Values.Add(smallest)
      End Sub

      ''' <summary> item will be associated with Values >= threshold</summary>
      ''' <exception cref="System.ArgumentException">ThresholdDictionary already contains this threshold</exception>
      Public Function Add(ByVal threshold As TKey, ByVal item As TValue) As ThresholdDictionary(Of TKey, TValue)
         Dim i = _Keys.BinarySearch(threshold) Xor -1
         If i < 0 Then
#If DEBUG Then
            Stop 'while Startup VS doesn't perform a Break-Point - so in Debug-Mode break manually
#End If
            Throw New ArgumentException(String.Concat(Me.GetType.Name, " already contains this threshold: ", threshold))
         End If
         _Keys.Insert(i, threshold)
         _Values.Insert(i + 1, item)
         Return Me
      End Function

      Default Public ReadOnly Property Item(ByVal key As TKey) As TValue
         Get
            Dim i = _Keys.BinarySearch(key)
            Return _Values(If(i < 0, Not i, i + 1))
         End Get
      End Property

      Private Function GetEnumerator() As Collections.IEnumerator Implements Collections.IEnumerable.GetEnumerator
         Return _Values.GetEnumerator
      End Function

      Public Function GetEnumerator1() As Collections.Generic.IEnumerator(Of TValue) Implements Collections.Generic.IEnumerable(Of TValue).GetEnumerator
         Return _Values.GetEnumerator
      End Function
   End Class

End Namespace