﻿Imports System.ComponentModel
Imports System.Collections
Imports System.Diagnostics
Imports System.Collections.Generic
Imports System.Windows.Media
Imports System.Runtime.CompilerServices

Imports DepProp = System.Windows.DependencyProperty
Imports UiData = System.Windows.UIPropertyMetadata
Imports FwData = System.Windows.FrameworkPropertyMetadata
Imports System.Reflection

Namespace System.Windows.Controls
   Public Enum DepPropOption
      AffectsArrange = 2
      AffectsMeasure = 1
      AffectsParentArrange = 8
      AffectsParentMeasure = 4
      AffectsRender = 16
      BindsTwoWayByDefault = 256
      [Inherits] = 32
      Journal = 1024
      None = 0
      NotDataBindable = 128
      OverridesInheritanceBehavior = 64
      SubPropertiesDoNotAffectRender = 2048
      IsAnimationProhibited = 4096
   End Enum

   Public Class DepProp

      Private Const _PublicDeclared As BindingFlags = BindingFlags.Public Or BindingFlags.DeclaredOnly

      Private Shared _tpOwner, _tpMember As Type
      Private Shared _MemberName As String, _Data As FwData, _Attache As Boolean

      Private Shared Sub PrepareMetaData( _
            ByVal defaultValue As Object, _
            ByVal changeCallback As PropertyChangedCallback, _
            ByVal coerceCallback As CoerceValueCallback, _
            ByVal options As DepPropOption)
         Dim animationProhibited = 0 <> (options And DepPropOption.IsAnimationProhibited)
         Dim opts = DirectCast(options And Not DepPropOption.IsAnimationProhibited, FrameworkPropertyMetadataOptions)
         _Data = New FwData(defaultValue, opts, changeCallback, coerceCallback, animationProhibited)
      End Sub

      Private Shared Sub Prepare( _
            ByVal defaultValue As Object, _
            ByVal changeCallback As PropertyChangedCallback, _
            ByVal coerceCallback As CoerceValueCallback, _
            ByVal options As DepPropOption)
         PrepareMetaData(defaultValue, changeCallback, coerceCallback, options)
         If changeCallback.Null Then Throw New ArgumentException("need a changedCallback to register")
         Dim mtd = changeCallback.Method
         _tpOwner = mtd.DeclaringType
         Dim cbName = mtd.Name
         Dim nameLengthSelector = Function(x As MemberInfo) x.Name.Length
         'gesucht wird die Objekt-Property mit dem längsten Namen, links-identisch zum Namen des Callbacks
         With (From p In _tpOwner.GetProperties(_PublicDeclared Or BindingFlags.Instance) _
               Where cbName.StartsWith(p.Name, StringComparison.Ordinal) _
               Select p).Extremum(nameLengthSelector)
            If .HasValue Then
               _Attache = False : _MemberName = .Max.Name : _tpMember = .Max.PropertyType
               Return
            End If
         End With
         'gesucht wird die statische Get-Methode mit dem längsten Namen, ab dem 3.zeichen
         ' links-identisch zum Namen des Callbacks
         With (From m In _tpOwner.GetMethods(_PublicDeclared Or BindingFlags.Static) _
               Let pName = m.Name _
               Where pName.StartsWith("Get", StringComparison.Ordinal) _
               Where cbName.StartsWith(pName.Substring(3), StringComparison.Ordinal) _
               Select m).Extremum(nameLengthSelector)
            If Not .HasValue Then Throw New TypeLoadException(String.Concat( _
               "no namely to '", cbName, "' associatable property found in class '", _tpOwner.Name, "'"))
            _Attache = True : _MemberName = .Max.Name.Substring(3) : _tpMember = .Max.ReturnType
         End With
      End Sub

      Private Shared Sub Prepare(ByVal tpOwner As Type, ByVal name As String, _
            ByVal defaultValue As Object, _
            ByVal changeCallback As PropertyChangedCallback, _
            ByVal coerceCallback As CoerceValueCallback, _
            ByVal options As DepPropOption)
         PrepareMetaData(defaultValue, changeCallback, coerceCallback, options)
         _tpOwner = tpOwner
         If name.StartsWith("Get", StringComparison.Ordinal) Then
            _Attache = True
            _MemberName = name.Substring(3)
            _tpMember = If(defaultValue IsNot Nothing, defaultValue.GetType, _
               _tpOwner.GetMethod(name, _PublicDeclared Or BindingFlags.Static).ReturnType)
         Else
            _Attache = False
            _MemberName = name
            _tpMember = If(defaultValue IsNot Nothing, defaultValue.GetType, _
               _tpOwner.GetProperty(name, _PublicDeclared Or BindingFlags.Instance).PropertyType)
         End If
      End Sub

      Public Sub New(ByVal tpOwner As Type, ByVal name As String, Optional ByVal options As DepPropOption = DepPropOption.None)
         Prepare(tpOwner, name, Nothing, Nothing, Nothing, options)
      End Sub
      Public Sub New(ByVal defaultValue As Object, ByVal tpOwner As Type, ByVal name As String, Optional ByVal options As DepPropOption = DepPropOption.None)
         Prepare(tpOwner, name, defaultValue, Nothing, Nothing, options)
      End Sub
      Public Sub New(ByVal changeCallback As PropertyChangedCallback, Optional ByVal options As DepPropOption = DepPropOption.None)
         Prepare(Nothing, changeCallback, Nothing, options)
      End Sub
      Public Sub New(ByVal defaultValue As Object, ByVal changeCallback As PropertyChangedCallback, Optional ByVal options As DepPropOption = DepPropOption.None)
         Prepare(defaultValue, changeCallback, Nothing, options)
      End Sub
      Public Sub New(ByVal changeCallback As PropertyChangedCallback, ByVal coerceCallback As CoerceValueCallback, Optional ByVal options As DepPropOption = DepPropOption.None)
         Prepare(changeCallback, Nothing, coerceCallback, options)
      End Sub
      Public Sub New(ByVal defaultValue As Object, ByVal changeCallback As PropertyChangedCallback, ByVal coerceCallback As CoerceValueCallback, Optional ByVal options As DepPropOption = DepPropOption.None)
         Prepare(defaultValue, changeCallback, coerceCallback, options)
      End Sub

      Public Shared Widening Operator CType(ByVal mdata As DepProp) As DependencyProperty
         If _Attache Then Return DependencyProperty.RegisterAttached(_MemberName, _tpMember, _tpOwner, _Data)
         Return DependencyProperty.Register(_MemberName, _tpMember, _tpOwner, _Data)
      End Operator

   End Class

End Namespace