﻿Imports System.ComponentModel

Namespace System.Windows.Controls

   Public Class GridEx : Inherits Grid

      Public Shared ReadOnly RangeProperty As DependencyProperty _
         = New DepProp("a1", AddressOf RangeCallback)
      <Description("sample: ""a1 b2"" would define cells(0,0) with column-/row-span(2,2)")> _
      <AttachedPropertyBrowsableForChildren()> _
      <Category("Layout")> _
      Public Shared Function GetRange(ByVal target As UIElement) As String
         Return DirectCast(target.GetValue(RangeProperty), String)
      End Function
      <AttachedPropertyBrowsableForChildren()> _
      <Category("Layout")> _
      Public Shared Sub SetRange(ByVal target As UIElement, ByVal value As String)
         target.SetValue(RangeProperty, value)
      End Sub

      Private Shared Sub RangeCallback( _
            ByVal sender As DependencyObject, ByVal e As DependencyPropertyChangedEventArgs)
         Dim s = DirectCast(If(e.NewValue, e.Property.GetMetadata(GetType(GridEx)).DefaultValue), String)
         Dim splits = s.ToLower.Split({" "c}, _
            StringSplitOptions.RemoveEmptyEntries)
         Dim xy = Parse(splits(0))
         Dim wh = If(splits.Length > 1, Parse(splits(1)), {-1, -1})
         For i = 0 To 1
            If xy(i) < 0 Then
               xy(i) = 0
               If wh(i) < 0 Then wh(i) = Short.MaxValue
            Else
               If wh(i) < 0 Then wh(i) = 0
            End If
            wh(i) += 1
         Next
         sender.SetValue(ColumnProperty, xy(0))
         sender.SetValue(RowProperty, xy(1))
         sender.SetValue(ColumnSpanProperty, wh(0))
         sender.SetValue(RowSpanProperty, wh(1))
      End Sub
      Private Shared Function Parse(ByVal s As String) As Integer()
         Dim x = AscW(s) - 97
         Dim y = 0
         Integer.TryParse(s.Substring(If(x < 0, 0, 1)), y)
         Return {x, y - 1}
      End Function
      Public Overrides Sub EndInit()
         For Each rw In RowDefinitions
            With rw
               If .Height.IsAuto AndAlso .MinHeight = 0 Then .MinHeight = 15
            End With
         Next
         For Each rw In ColumnDefinitions
            With rw
               If .Width.IsAuto AndAlso .MinWidth = 0 Then .MinWidth = 15
            End With
         Next
         MyBase.EndInit()
      End Sub
   End Class
End Namespace
