﻿Imports System.ComponentModel

Namespace System.Windows.Input
   Public Class RelayCommand : Inherits CommandBase
      Private ReadOnly _execute As Action
      Private ReadOnly _canExecute As Func(Of Boolean) = Function() Enabled
      <DebuggerStepThrough()> _
      Public Sub New(ByVal execute As Action, Optional ByVal canExecute As Func(Of Boolean) = Nothing)
         If execute Is Nothing Then Throw New ArgumentNullException("execute")
         Me._execute = execute
         If canExecute.NotNull Then Me._canExecute = canExecute
      End Sub
      <DebuggerStepThrough()> _
      Public Overrides Function CanExecute(ByVal parameter As Object) As Boolean
         Return _canExecute()
      End Function
      <DebuggerStepThrough()> _
      Public Overrides Sub Execute(ByVal parameter As Object)
         Me._execute()
      End Sub
   End Class

   Public Class RelayCommand(Of T) : Inherits CommandBase
      Private ReadOnly _execute As Action(Of T)
      Private ReadOnly _canExecute As Predicate(Of T) = Function(itm As T) Enabled
      <DebuggerStepThrough()> _
      Public Sub New(ByVal execute As Action(Of T), Optional ByVal canExecute As Predicate(Of T) = Nothing)
         If execute Is Nothing Then Throw New ArgumentNullException("execute")
         Me._execute = execute
         If canExecute.NotNull Then Me._canExecute = canExecute
      End Sub
      <DebuggerStepThrough()> _
      Public Overrides Function CanExecute(ByVal parameter As Object) As Boolean
         Return _canExecute(DirectCast(parameter, T))
      End Function
      <DebuggerStepThrough()> _
      Public Overrides Sub Execute(ByVal parameter As Object)
         Me._execute(DirectCast(parameter, T))
      End Sub
   End Class

   Public MustInherit Class CommandBase : Inherits NotifyPropertyChanged : Implements ICommand

      Private EventDisabled As New Counter 'verhindert Ereigniskette beim gegenseitigen Aufruf von CanExecute und Setter der Enabled-Property

      'das CanExecuteChanged-Event vereinigt sich mit dem statischen CommandManager.RequerySuggested-Event. Eiglich müsste es LaunchRequery heißen oderso 
      Public Custom Event CanExecuteChanged As EventHandler Implements ICommand.CanExecuteChanged
         AddHandler(ByVal value As EventHandler)
            AddHandler CommandManager.RequerySuggested, value
         End AddHandler
         RemoveHandler(ByVal value As EventHandler)
            RemoveHandler CommandManager.RequerySuggested, value
         End RemoveHandler
         RaiseEvent(ByVal sender As Object, ByVal e As EventArgs)
            CommandManager.InvalidateRequerySuggested()
         End RaiseEvent
      End Event

      Public Shared ReadOnly EnabledChangedArgs As New PropertyChangedEventArgs("Enabled")

      Public MustOverride Sub Execute(ByVal parameter As Object) Implements ICommand.Execute
      Public MustOverride Function CanExecute(ByVal parameter As Object) As Boolean
      Private _Enabled As Boolean = True
      Public Property Enabled() As Boolean
         Get
            Return _Enabled
         End Get
         Set(ByVal value As Boolean)
            If ChangePropIfDifferent(value, _Enabled, EnabledChangedArgs) Then
               If EventDisabled.Up = 0 Then RaiseEvent CanExecuteChanged(Me, EventArgs.Empty)
               EventDisabled.Down()
            End If
         End Set
      End Property
      Private Function ICanExecute(ByVal parameter As Object) As Boolean Implements ICommand.CanExecute
         If EventDisabled.Up = 0 Then Enabled = CanExecute(parameter)
         EventDisabled.Down()
         Return _Enabled
      End Function
   End Class
End Namespace
